#include <TMB.hpp>
template<class Type>
Type objective_function<Type>::operator() ()
{
  using namespace density;
  
  DATA_FACTOR(j); // female id
  DATA_FACTOR(k); // territory id
  DATA_FACTOR(t); // year id
  DATA_VECTOR(z);
  DATA_MATRIX(y);
  DATA_VECTOR(temp);
  DATA_VECTOR(April_temp);
  DATA_VECTOR(dens);
  DATA_VECTOR(altitude);
  DATA_VECTOR(tt); // year 
  DATA_FACTOR(a); // age
  DATA_VECTOR(offset); 
  DATA_VECTOR(survival); // mother survival (0 or 1)
  DATA_VECTOR(brood); // brood number
  DATA_VECTOR(eggs); // total # of eggs
  
  PARAMETER_VECTOR(beta_0);
  PARAMETER_VECTOR(beta_age); // linear effect of age
  PARAMETER_VECTOR(beta_clutchsize);
  PARAMETER_VECTOR(beta_eggs); 
  PARAMETER_VECTOR(beta_temp);
  PARAMETER_VECTOR(beta_dens);
  PARAMETER_VECTOR(beta_altitude);
  PARAMETER_VECTOR(beta_clutchsize_layingdate);
  PARAMETER_MATRIX(u);
  PARAMETER_VECTOR(v);
  PARAMETER_VECTOR(w);
  PARAMETER_MATRIX(x);
  PARAMETER_VECTOR(log_sigma); vector<Type> sigma = exp(log_sigma); ADREPORT(sigma);
  PARAMETER_VECTOR(log_tau);   vector<Type> tau   = exp(log_tau);   ADREPORT(tau);
  PARAMETER_VECTOR(log_kappa); vector<Type> kappa = exp(log_kappa); ADREPORT(kappa);
  PARAMETER_VECTOR(log_gamma); vector<Type> gamma = 1 + 1e-10 + exp(log_gamma); ADREPORT(gamma);
  PARAMETER_VECTOR(log_nu);    vector<Type> nu = exp(log_nu);       ADREPORT(nu);
  
  PARAMETER_VECTOR(theta_0);
  PARAMETER_VECTOR(theta_temp);
  PARAMETER_VECTOR(theta_April_temp);
  PARAMETER_VECTOR(theta_dens);
  PARAMETER_VECTOR(theta_altitude);
  PARAMETER_VECTOR(theta_tt);
  PARAMETER_VECTOR(theta_clutchsize);
  PARAMETER(log_sd_zeta); Type sd_zeta = exp(log_sd_zeta); ADREPORT(sd_zeta);
  
  PARAMETER_VECTOR(log_omega); vector<Type> omega=exp(log_omega); ADREPORT(omega);

  Type nll = 0;

  // Vector auto regressive process for u_t, zeta_t
  int d = 2; // Dimension of the process
  PARAMETER_VECTOR(phi); // Elements of Phi of VAR(1) process
  PARAMETER(rho); // correlation of white noise in the VAR(1) process
  matrix<Type> Phi(d,d);                  // Put elements of phi into Phi
  Phi(0,0) = phi(0); Phi(0,1) = phi(1);
  Phi(1,0) = phi(2); Phi(1,1) = phi(3);
  vector<Type> vecSigma(d*d);             // vec(White noise variance matrix)
  vecSigma(0) = 1;           vecSigma(1) = rho;
  vecSigma(2) = vecSigma(1); vecSigma(3) = 1;
  matrix<Type> Sigma(2,2);                // white noise variance matrix
  Sigma(0,0) = vecSigma(0); Sigma(0,1) = vecSigma(1);
  Sigma(1,0) = vecSigma(2); Sigma(1,1) = vecSigma(3);
  matrix<Type> A(d*d,d*d);                // minus kronecker(Phi,Phi)
  for (int i=0; i<d; i++)
    for (int j=0; j<d; j++)
      for (int k=0; k<d; k++)
        for (int l=0; l<d; l++)
          A(i*d+k, j*d+l) = -Phi(i,j)*Phi(k,l);
  for (int i=0; i<d*d; i++)               // plus the identity matrix
    A(i,i) += 1;
  matrix<Type> Ainv = A.inverse();        // Computes inverse of A = I - Phi kronecker Phi
  vector<Type> vecGamma0 = Ainv*vecSigma; // vec(Stationary covariance matrix)
  matrix<Type> Gamma0(d,d);
  for (int i=0; i<d; i++)
    for (int j=0; j<d; j++)
      Gamma0(i,j) = vecGamma0(i+j*d);
  // Rescaling such such that u_{t,1} is standard normal and SD(u_{t,2})=sd_zeta
  vector<Type> sds(2);
  sds(0) = 1      /sqrt(Gamma0(0,0));
  sds(1) = sd_zeta/sqrt(Gamma0(1,1));
  // always let the following hang around (should integrate to one if disconnected from rest of model)
  nll += VECSCALE(MVNORM(Gamma0),sds)(u.row(0));
  for (int t=1; t<u.rows(); t++) {
    vector<Type> ut = u.row(t);
    vector<Type> utminus1 = u.row(t-1);
    vector<Type> Phix = Phi*utminus1;
    vector<Type> error = ut - Phix;
    nll += VECSCALE(MVNORM(Sigma),sds)(error); // Addition here since MVNORM returns the negative of the contribution
  }
  
  if (CppAD::Variable(v(0)))
    nll -= sum(dnorm(v, Type(0), Type(1), true)); // iid N(0,1) territory effects
  
  if (CppAD::Variable(w(0)))
    nll -= sum(dnorm(w, Type(0), Type(1), true)); // iid N(0,1) female id effects
  
  // age effects modelled as second order random walk
  // control magnitude via episode specific nu[s)
  for (int s=0; s<3; s++) {
    if (CppAD::Variable(log_nu(s)))   // model age effects as RW2
      for (int a=2; a<x.rows(); a++)
        nll -= dnorm(x(a,s), 2*x(a-1,s) - x(a-2,s), nu(s), true);
    else                              // dummy Gaussian density if x[,s] is disconnected
      for (int a=0; a<x.rows(); a++)
        nll -= dnorm(x(a,s), Type(0), Type(1), true);
  }
  
  // loop through all clutches and episodes
  for (int i=0; i<y.rows(); i++) {
    for (int s=0; s<3; s++) { // loop through all episodes including mothers survival (episode 3)
      Type eta                             \
      = beta_0[s]                          \
      + beta_clutchsize[s]*y(i,0)          \
      + beta_eggs[s]*eggs[i]               \
      + beta_temp[s]*temp[i]               \
      + beta_dens[s]*dens[i]               \
      + beta_altitude[s]*altitude[i]       \
      + beta_clutchsize_layingdate[s]*y(i,0)*z(i) \
      + offset[s]                          \
      + beta_age[s]*a[i]                   \
      ;
      if (CppAD::Variable(log_sigma(s)))
        eta += sigma[s]*u(t[i],0);  // First columnn is effect on survival
      if (CppAD::Variable(log_tau(s)))
        eta += tau[s]  *v[k[i]];
      if (CppAD::Variable(log_kappa(s)))
        eta += kappa[s]*w[j[i]];             
      if (CppAD::Variable(log_nu(s)))
        eta += x(a[i],s);
      
      if (CppAD::Variable(theta_0(s))) { // Do not include stab. sel on laying date if intercept of optimum is NA
        Type eta_theta
        = theta_0[s]                       \
        + theta_dens[s]*dens[i]            \
        + theta_temp[s]*temp[i]            \
        + theta_April_temp[s]*April_temp[i]\
        + theta_altitude[s]*altitude[i]    \
        + theta_tt[s]*tt[i]    \
        + theta_clutchsize[s]*y(i,0);
        if (CppAD::Variable(log_sd_zeta))
          eta_theta += u(t[i],1);
        eta += pow((z[i] - eta_theta)/omega[s],2)/2;
      }
      
      if (s<2) {
        // Parameters of beta distribution parameterized in terms of
        // eta=cloglog(1-p) and dispersion parameter gamma (variance relative to binomial)
        Type p     = exp(-exp(eta));
        Type alpha = p      *(y(i,s) - 1)/(gamma(s) - 1) + 1e-100;
        Type beta  = (1 - p)*(y(i,s) - 1)/(gamma(s) - 1) + 1e-100;
        // log of Beta binomial point mass function
        nll -=
          lfactorial(y(i,s)) - lfactorial(y(i,s+1)) - lfactorial(y(i,s) - y(i,s+1)) \
        + lgamma(alpha + y(i,s+1)) + lgamma(beta + y(i,s) - y(i,s+1)) - lgamma(alpha + beta + y(i,s)) \
        - lgamma(alpha)            - lgamma(beta)                     + lgamma(alpha + beta);
      } else {
        if (brood(i)==1)
          nll -= dbinom(survival[i], Type(1), exp(-exp(eta)), true); // mother survival ("episode 3", ie. s=2)
      }
    }
  }
  
  return nll;
}
