# Function computing marginal survival integrating out a random effect with 
# standard deviation sigma that appears as a linear term in the linear predictor eta
marginalsurvival <- Vectorize(function(eta,sigma=exp(logsigma),logsigma=log(sigma)) {
  f <- function(u) {
    exp(-exp(eta+u))*dnorm(u,sd=sigma)
  }
  integrate(f, -5*sigma, 5*sigma)$value
})


subsetpar <- function(x,name)
  subset(x,names(x)==name)

rename <- function(x, name, newname) {
  pos <- match(name,names(x))
  names(x)[pos] <- newname
  x
}

# Refit the model, possibly with some modifications to map argument using previous
# estimates as starting values. 
# 
# Arguments:
#   model: A list representing a previously fitted model with components
#     estimates:  Estimates from a previous model fit in same format as parameters
#     parameters: Parameter values of restricted models
#     map:        Specifies which parameters were estimated or restricted (input the MakeADFun)
#     args.MakeADFun: Additional arguments to MakeADFun (excluding map and parameters)
#     args.optim: Additional argument used to fit the model
#     sdreport:   Standard errors etc...
#     description: A string of latex describing the model
#
#   changemap:  A list containing changes we want in the map-argument
#
#   startsmart: If TRUE use previous estimates if available as startingvalues
#
#   cv: coefficient of variation of lognormal changes in all starting values
#
#   seed: If not NULL, set.seed(seed) is called before adding noise to 
#     initial values.  The seed is then restored to its previous value in the global environment
#     Use this to make the model fitting reproducable.
#
#   description : A string of latex describing the model.  Typically how it differs from some best
#     model
# 
# Value: 
#    A list with the same format as the model argument containing the new model
#   
update.tmbmodel <- function(model, changemap=NULL, startsmart=TRUE, cv=NULL, seed=NULL, description="") {
  newmodel <- model
  
  # Compute map for newmodel
  for (parname in names(changemap)) {
    newmodel$map[[parname]] <- changemap[[parname]]
  }
  
  # Compute starting values for newmodel in part using estimates from model
  start <- model$parameters # use parameter values under restricted model as default starting value
  if (startsmart)
    if (!is.null(model$estimates)) { # if previous estimates are available
      for (parname in names(start)) {
        if (parname %in% names(newmodel$map)) {
          estimated.subset <- !is.na(newmodel$map[[parname]])
          # then for elements that we want to estimate, use previous estimate as starting value
          start[[parname]][estimated.subset] <- model$estimates[[parname]][estimated.subset]
        } else {
          start[[parname]] <- model$estimates[[parname]]
        }
      }
    }
  
  
  # Make new objective function
  args.MakeADFun <- model$args.MakeADFun
  args.MakeADFun$parameters <- start 
  args.MakeADFun$map <- newmodel$map
  obj <- do.call("MakeADFun", args.MakeADFun)
  
  # Add any extra arguments and call optimizer
  for (varname in names(newmodel$args.optim))
    obj[[varname]] <- model$args.optim[[varname]]
  if (!is.null(cv)) {
    if (is.null(seed)) {
      oldseed <- .Random.seed
      set.seed(seed)
    }
    obj$par <- obj$par*rlnorm(length(obj$par),sd=cv)
    if (is.null(seed)) 
      assign(".Random.seed", oldseed, .GlobalEnv)
  }
  if (model$optimizer=="nlminb") 
    opt <- nlminb(obj$par, obj$fn, obj$gr)
  else
    opt <- do.call(model$optimizer,obj)
  
  newmodel$sdreport <- sdreport(obj)
  newmodel$opt <- opt
  newmodel$aic <-  2*length(opt$par) - 2*(-opt$value) # 2p - 2l_max
  newmodel$obj <- obj
  newmodel$description <- description
  
  # Put all estimated and constrained parameters into estimates
  newmodel$estimates <- newmodel$parameters 
  for (parname in names(newmodel$parameters)) {
    # parameter in map argument, overwrite elements that are non-missing according to each level in factor
    if (parname %in% names(newmodel$map)) { 
      estimated.subset <- !is.na(newmodel$map[[parname]])
      newmodel$estimates[[parname]][estimated.subset] <- subsetpar(opt$par,parname)[newmodel$map[[parname]]][estimated.subset]
    } else { # parameter not in map argument 
      if (parname %in% names(opt$par)) { # then if is estimated extract those from opt$par
        newmodel$estimates[[parname]] <- subsetpar(opt$par,parname) 
        names(newmodel$estimates[[parname]]) <- NULL
      } else { # this must be a random effect
        newmodel$estimates[[parname]] <- subsetpar(summary(newmodel$sdreport,"random")[,1], parname) 
        names(newmodel$estimates[[parname]]) <- NULL
        dim(newmodel$estimates[[parname]]) <- dim(newmodel$parameters[[parname]])
      }
    }
  }
  newmodel
}

# print method for tmbmodel objects
print.tmbmodel <- function(model) {
  fixed <- summary(model$sdreport,"fixed")
  report <- summary(model$sdreport,"report")
  tmp <- rbind(fixed,report)
  tmp <- cbind(tmp, z=tmp[,"Estimate"]/tmp[,"Std. Error"])
  tmp <- cbind(tmp, "P(|Z|>|z|)"=2*pnorm(abs(tmp[,"z"]),lower.tail = FALSE))
  tmp <- cbind(tmp, Gradient=c(model$sdreport$gradient.fixed, rep(NA,nrow(report))))
  tmp <- cbind(tmp, "Scaled Grad."=tmp[,"Gradient"]*tmp[,"Estimate"])
  subset <- tmp[,"Std. Error"]!=0
  subset[is.na(subset)] <- TRUE
  tmp <- tmp[subset,]
  print(signif(tmp,digits=4))
  cat("Maximum scaled outer gradient",max(tmp[1:nrow(fixed),"Scaled Grad."]),"\n")
  cat("smallest eigenvalue = ",min(eigen(model$opt$hessian)$val),"\n")
  cat("convergence = ",model$opt$convergence,"\n")
  cat("aic = ",model$aic,"\n")
}

# Matrix containing latex code and explanation of each parameter
partable <- read.table("parameter-descriptions.txt",header=TRUE,sep=";",as.is=TRUE,strip.white=TRUE)
# Generate table of parameter estimates for model object returned by update.tmbmodel function
parametertable <- function(model,file=NULL) {
  if (!is.null(file)) 
    sink(file)
  cat("\\begin{tabular}{l l |l l l |l} \n")
  cat("& & \\multicolumn{3}{c|}{Selection episode} \\\\ \n")
  cat("\\multicolumn{2}{c|}{Parameters} & Egg to fledgling & Fledgling to recruit & Adult female\\\\ \n")
  cat("Meaning & Symbol & $s=1$ & $s=2$ & $s=3$  & Unit \\\\ \n")
  cat("\\midrule\n")
  
  catpm <- function(est) {
    if (is.na(est[1]))
      cat(" & $0$")
    else
      cat(" & $",format(est["Estimate"],nsmall=nsmall,scientific = nsmall),
          "\\pm",format(est["Std. Error"],nsmall=nsmall,scientific = nsmall),"$")
  }
  estimates <- summary(model$sdreport,c("fixed","report"))
  for (parname in rownames(partable)) {      # loop through all possible parameters
    if (parname=="midrule") {
      cat("\\midrule\n")
    } else {
      parlength <- length(model$parameters[[parname]])
      if (is.null(model$map[[parname]]))          # if not included in map argument 
        map <- 1:parlength                        # all 3 (or 2) levels are in effect estimated
      else                                        # otherwise
        map <- model$map[[parname]]               # look at the map argument
      if (sum(!is.na(map))>0) {                   # if at least one level is estimated include in table
        if (substr(parname,1,5)=="logit") {         # estimates through ADREPORT
          name <- substring(parname,7)            # estimates appear three times in "report"
          subset <- rownames(estimates)==name 
          est <- estimates[subset,,drop=FALSE]
        } else if (substr(parname,1,3)=="log") {         # estimates through ADREPORT
          name <- substring(parname,5)            # estimates appear three times in "report"
          subset <- rownames(estimates)==name 
          est <- estimates[subset,,drop=FALSE]
        } else { # the parameter appears in fixed part once for each estimated level 
          name <- parname
          subset <- rownames(estimates)==name
          est <- estimates[subset,,drop=FALSE][model$map[[parname]],,drop=FALSE]
        }
        est[est[,"Std. Error"]==0,] <- NA # Hack to make things work also for parameters that defaults to non-zero values
        digits <- max(0,2-ceiling(log10(min(est[,"Std. Error"], na.rm=TRUE))))
        est <- round(est,digits)
        nsmall <- max(0,digits)
        cat(partable[parname,"meaning"], "& $\\hat",partable[parname,"symbol"],"$ ",sep="")
        if (name=="phi") {
          catpm(est[1,])
          catpm(est[2,]) 
          cat("&&\\\\ \n &")
          catpm(est[3,]) 
          catpm(est[4,])
          cat("&")
        } else {
          for (s in 1:3) {
            if (s<=parlength) {
              catpm(est[s,])
            } else
              cat(" &")
          }
        }
        cat("& \\si{",partable[parname,"unit"],"}")
        cat("\\\\ \n")
      }
    }
  }
  
  cat("\\bottomrule\n")
  cat("\\end{tabular}\n")
  if (!is.null(file)) 
    sink()
}

# AIC method for objects of class tmbmodel 
AIC.tmbmodel <- function(model) {
  2*model$opt$val + 2*length(model$opt$par)
}
logLik.tmbmodel <- function(model) {
  -model$opt$val
}
anova.tmbmodel <- function(model0,model1) {
  lambda <- abs(2*(logLik(model1)-logLik(model0)))
  df <- abs(length(model1$opt$par)-length(model0$opt$par))
  cat("LR =",lambda," df=",df," p-value =",pchisq(lambda,df,lower.tail=FALSE),"\n")
}

# Produce a table of different models, each model has its own label (modelno counter)
# that can be referred to with \ref
#
# Arguments:
#    models : A list of models (of class tmbmodel) returned by update.tmbmodel
#    bestmodel : The name of the list component containing the bestmodel
#    file: File to sink output to
#    exclude: Another list of models that we possibly don't want to include in the table
#    showkeys: Print the latex cross-referencing key next to the label
#
aictable <- function(models,best="best",file=NULL,exclude=NULL,showkeys=FALSE) {
  aic.best <- AIC(models[[best]])
  p0 <- length(models[[best]]$opt$par)
  if (!is.null(file)) sink(file) 
  cat("\\begin{tabular}{lrrl}\n")
  cat(" & $\\Delta\\mathrm{AIC}$ & $\\Delta p$ & Description\\\\ \n")
  cat("\\hline\n")
  for (name in names(models)) if (!name %in% names(exclude)) {
    delta.aic <- AIC(models[[name]]) - aic.best
    delta.p <- length(models[[name]]$opt$par) - p0
    space <- paste(rep(" ",10-nchar(name)),collapse = "")
    cat("\\model{",name,"}",if (showkeys) paste("\\tiny{",name,"}",collapse="",sep=""),space," & ",round(delta.aic,2)," & ",delta.p," & ",models[[name]]$description,sep="")
    cat("\\\\ \n")
  }
  cat("\\hline \n")
  cat("\\end{tabular}\n")
  if (!is.null(file)) sink()
}

# Compute mean fitness as function of clutchsize given the linear predictors for two
# episodes averaging over the distribution of the territory and female id random effects
# v and w.
#
# Arguments:
#   x :        Clutchsize
#   eta.part : Vector containing the rest of the linear predictor for episode 1 and 2
#              including the offset and quadratic term but excluding the term for the effect of clutchsize 
#              and the territory and female id random effect
#   beta :     Vector containing the regression coefficient for clutchsize for 
#              episode 1 and 2
#   tau :      standard deviations of territory random effect for episode 1 and 2
#   kappa :    standard deviations of female id random effect for episode 1 and 2
#  
# Value:
#
#   Mean contribution to fitness from the cluth, that is, clutch size multiplied
#   product of survival propbabilities
#
wbar <- function(x, eta.part, beta, tau, kappa) { 
  f <- function(u) {
    x*exp(-sum(exp(eta.part + beta*x + tau*u[1] + kappa*u[2])))*prod(dnorm(u)) 
  }
  cubature::hcubature(f, rep(-3, 2), rep(3, 2))$integral
}


# Compute optimal clutchsize given of the linear predictors for episode 1 and 2
# and regression coefficients for effects of clutchsize
# averaging over the distribution of the territory and female id random effects
# v and w.
#
# Arguments:
#   eta.part, beta, tau, kappa : Same as for wbar
#  
# Value:
#
#   Optimal clutchsize
#
optimalclutchsize <- function(eta.part, beta, tau, kappa) { 
  g <- function(u, x) {
    exp(sum(-(exp(eta.part + beta*x + tau*u[1] + kappa*u[2]))))* 
      (1-x*sum(beta*exp(eta.part + beta*x + tau*u[1] + kappa*u[2])))* 
      prod(dnorm(u))
  }
  h <- function(x) {
    cubature::hcubature(g, rep(-3, 2), rep(3, 2), x=x, tol=1e-3)$integral
  }
  uniroot(h, c(0,20))$root
}


############ New code computing optimas and associated Gaussian approximations for a given model #############
optimas <- function(model, dataset=data) {
  dens <- tapply(dipperdata$N,dipperdata[,1],mean)-mean(tapply(dipperdata$N,dipperdata[,1],mean))
  temp <- tapply(dipperdata$temp,dipperdata[,1],mean)-mean(tapply(dipperdata$temp,dipperdata[,1],mean))
  tt <- 1979:2013; tt <- tt - mean(tt)
  n.years <- length(dens)
  with(model$estimates,{
    opt <- matrix(NA, n.years, 4) # Matrix containing the joint optimas
    colnames(opt) <- c("cs","ld","ld-s-1","ld-s-2")
    hessians <- array(dim=c(2,2,n.years)) # Hessian matrices at each optims
    omega <- matrix(NA, 2, n.years) # Matrix containing widths of Gaussian fitness function approximation 
    
    # optimal laying dates with respect to episode 1 and 2
    for (s in 1:2) {
      opt[,2+s] <- theta_0[s] + theta_dens[s]*dens + theta_temp[s]*temp + theta_tt[s]*tt + u[,2] # Note that u[,2] contains zeta_t
    }
    
    # Compute the log of overall mean fitness for episodes 1 and 2 combined with respect to clutchsize z[1] and laying date z[2]
    lnwbar <- function(z) { 
      w <- function(random) { # individual fitness function for episodes 1 and 2 combined conditional on random territory and female id effects
        eta.part <- numeric(2)
        for (s in 1:2) {
          eta.part[s] <- 
            # eta_beta (max survival part)
            beta_0[s] +    
            beta_clutchsize[s]*z[1] +
            beta_temp[s]*temp[t] + 
            beta_dens[s]*dens[t] + 
            exp(log_tau[s])*random[1] +
            exp(log_kappa[s])*random[2] +
            exp(log_sigma[s])*u[t,1] +
            # eta_theta (laying date part)
            1/(2*exp(log_omega[s])^2)*(z[2] - opt[t,2+s])^2 + 
            # offset term
            dataset$offset[s] 
        }
        z[1]*exp(-sum(exp(eta.part)))
      }
      wbar <- cubature::hcubature(w, rep(-3, 2), rep(3, 2))$integral
      log(wbar)
    }
    
    # optimal clutchsizes assuming that laying dates are at the optimum
    for (t in 1:n.years) {
      fit <- optim(c(5,40), lnwbar, control=list(fnscale=-1), hessian=TRUE)
      opt[t,c("cs","ld")] <- fit$par
      hessians[,,t] <- fit$hessian
      omega[,t] <- sqrt(diag(solve(-hessians[,,t])))
      cat("t = ",t,"$convergence = ",fit$convergence,"\n")
    }
    
    # Return the optimas, the omega's and the all hessian matrices
    tmp <- list(optima=opt, omega=omega, hessian=hessians)
    class(tmp) <- "optimas"
    tmp
  })
}


############ Function for plotting fluctuating optimums with the best model AND the simple one #########
fig.optimas <- function(opt, episodes=TRUE)
{
  n.models <- length(opt)
  par(mar=c(4,4,0.2,0.2))
  par(mfrow=c(2,n.models))
  
  completedata <- read.csv("Complete data set_with territory.txt",header=T,sep=";",fileEncoding="utf8")
  years <- 1979:2013
  layout(rbind(1:n.models,n.models+1:n.models),widths=1,heights=c(3,4))
  
  # clutchsizes
  p <- with(completedata, table(completedata[,1],Egg_ok))
  p <- p/apply(p,1,sum)
  for (i in 1:n.models) {
    par(mar=c(1,4,.2,.2))
    plot(NA,xlim=c(1979,2013),ylab="Clutch size",xlab="",frame=F,ylim=c(0,8),axes=F)
    axis(2)
    egg<-tapply(completedata$Egg_ok,completedata[,1],mean,na.rm=T)
    lines(egg~sort(unique(completedata[,1])),col="darkgrey",lwd=2)
    for (cs in 0:7) {
      points(years, rep(cs,35), pch=16, col="grey", cex=2*sqrt(p[,as.character(cs)]))
    }
    lines(years, opt[[i]]$optima[,1],lty=1)
    legend("topleft",legend=LETTERS[-1+2*i],bty="n")
  }
  
  
  # layingdates
  p <- with(completedata, table(completedata[,1],laying))
  p <- p/apply(p,1,sum)
  for (i in 1:n.models) {
    par(mar=c(4,4,.2,.2))
    plot(NA,xlim=c(1979,2013),ylab="Egg-laying date",xlab="Years",frame=F,ylim=c(0,100),axes=F)
    axis(1)
    axis.date(2)
    date<-tapply(completedata$laying,completedata[,1],mean,na.rm=T)
    lines(date~sort(unique(completedata[,1])),col="darkgrey",lwd=2)
    ll<-sort(unique(completedata$laying))
    for (j in 1:78) {
      points(years, rep(ll[j],35), pch=16, col="grey", cex=2*sqrt(p[,as.character(ll[j])]))
    }
    if (episodes) {
      for (s in 1:2) {
        lines(years, opt[[i]]$optima[,2+s],lty=i,col=c("red","green")[s])
      }
    }
    lines(years, opt[[i]]$optima[,4],col="green",lwd=4)
    lines(years, opt[[i]]$optima[,2],lty=1)
    legend("topleft",legend=LETTERS[2*i],bty="n")
  }
}



# Process data from eklima.  Compute date at which smoothed temperature
# passes a threshold temperature
#
# Arguments:
#   threshold: Threshold temperature
#   bw:        Bandwidth of the kernel regression 
#   plot:      If TRUE produce a plot showing everything
#
# Value: A data.frame containing the thresholds and the years (1991 is missing)
date.threshold <- function(threshold=5,bw=50, plot=TRUE) {
  eklima <- read.table("eklima-audnedal.txt",skip=38,header=TRUE,sep=";",dec=".",as.is=TRUE)
  eklima$TAM <- as.numeric(eklima$TAM)
  eklima <- eklima[!is.na(eklima$TAM),]
  date <- as.Date(eklima$Date, format="%d.%m.%Y")
  temp <- eklima$TAM
  station <- as.factor(eklima$St.no)
  
  est <- approxfun(ksmooth(date, temp, bandwidth = bw, kernel="normal"))
  f <- function(x) est(x) - threshold
  if (plot) par(mfcol=c(5,7))
  res <- NULL
  for (year in c(1979:1990,1992:2013)) {
    date.threshold <- uniroot(f,as.numeric(as.Date(paste(year,c("-01-20","-07-20"),sep=""))))$root
    res <- rbind(res, data.frame(year, date.threshold-(year-1970)*365.25))
    if (plot) {
      subset <- date >= as.Date(paste(year,"-01-01",sep="")) & date < as.Date(paste(year+1,"-01-01",sep=""))
      plot(date[subset], temp[subset], pch=".", ylim=c(-20,20), xlab="Date",ylab="temp")
      title(year)
      curve(est(x),add=TRUE)
      abline(v=date.threshold)  
      abline(h=threshold)
    }
  }
  res
}


############### wbar_full function -> recruitment (= clutch size * offspring survival)
wbar_full <- function(x, eld, eta.part, beta, num, denom, tau, kappa, off) { 
  f <- function(u) {
    x*exp(sum(off*-(exp(eta.part + beta*x + 
                          (eld-num)^2/denom
                        + tau*u[1] + kappa*u[2]))))*prod(dnorm(u)) 
  }
  cubature::hcubature(f, rep(-3, 2), rep(3, 2))$integral
}

################ wbar_full1 function -> survival only (remove clutch size)
wbar_full1 <- function(x, eld, eta.part, beta, num, denom, tau, kappa, off) { 
  f <- function(u) {
    exp(sum(off*-(exp(eta.part + beta*x + 
                        (eld-num)^2/denom
                      + tau*u[1] + kappa*u[2]))))*prod(dnorm(u)) 
  }
  cubature::hcubature(f, rep(-3, 2), rep(3, 2))$integral
}


############ Annual values of recruitment
annual_recruit<-function(mod)
{
  xx <- NULL
  dens <- tapply(dipperdata$N,dipperdata$Ar,mean)-mean(tapply(dipperdata$N,dipperdata$Ar,mean))
  temp <- tapply(dipperdata$temp,dipperdata$Ar,mean)-mean(tapply(dipperdata$temp,dipperdata$Ar,mean))
  altitude <-mean(dipperdata$HoH)
  ann<-sort(unique(dipperdata$Ar))
  
  for (t in 1:35) {
    eta.part <-  matrix(NA, nrow=2, ncol=35)
    num <- matrix(NA, nrow=2, ncol=35)
    denom <- matrix(NA, nrow=2, ncol=35)
    
    for (s in 1:2) {
      eta.part[s,] <-mod$estimates$beta_0[s]+
        mod$estimates$beta_temp[s]*temp+
        mod$estimates$beta_dens[s]*dens+
        mod$estimates$beta_altitude[s]*altitude+
        exp(mod$estimates$log_sigma[s])*mod$estimates$u      
      
      num[s,] <- mod$estimates$theta_0[1]+mod$estimates$theta_temp[1]*temp+mod$estimates$theta_dens[1]*dens+mod$estimates$theta_altitude[1]*altitude+mod$estimates$u[,2] 
      denom[s,] <- (2*exp((mod$estimates$log_omega[1]))^2)
    }
    xx[t] <-wbar_full(5, 55, eta.part[1:2,t], mod$estimates$beta_clutchsize[1:2], num[1:2,t], denom[1:2,t], exp(mod$estimates$log_tau[1:2]), exp(mod$estimates$log_kappa[1:2]) , c(22/365,343/365))
  } 
  plot(xx~ann, type="l", frame=F, ylab="Annual recruitment",xlab="Years")
}



############ Function for plotting mean recruitment as a function of clutch size and egg-laying date
# Fix density, altitude and temperature to their means (i.e. 0)
contour.recruitment<-function(mod)
{
  eta.part <-  NULL
  num <- NULL
  denom <- NULL
  xx<-matrix(NA, nrow=7, ncol=100)
  eldd<-seq(1,100)
  x<-seq(1,7)
  
  for (s in 1:2) {
    eta.part[s] <-mod$estimates$beta_0[s]
    num[s] <- mod$estimates$theta_0[s]
    denom[s] <- (2*exp((mod$estimates$log_omega[1]))^2)
  }
  
  for (i in seq_along(x)) {
    for (j in seq_along(eldd)) {
      xx[i,j] <- wbar_full(x[i], eldd[j], eta.part[1:2], mod$estimates$beta_clutchsize[1:2], num[1:2], denom[1:2], exp(mod$estimates$log_tau[1:2]), exp(mod$estimates$log_kappa[1:2]) , c(22/365,343/365))
    }
  } 
  contour(x,eldd, xx, ylab="",xlab="Clutch size", axes=F)
  axis(1)
  axis.date(labels=F)
  points(mean(completedata$Egg_ok,na.rm=T),mean(completedata$laying,na.rm=T),pch=3, col="red") # complete dataset
  #points(mean(dipperdata$Egg_ok,na.rm=T),mean(dipperdata$laying,na.rm=T),pch=3, col="blue") # our subset
}


############ Function for plotting mean survival on episode 1 as a function of clutch size and egg-laying date
contour.survival<-function(mod,s)
{
  num <- NULL
  denom <- NULL
  xx<-matrix(NA, nrow=7, ncol=100)
  eldd<-seq(1,100)
  x<-seq(1,7)
  eta.part <-mod$estimates$beta_0[s]
  num <- mod$estimates$theta_0[s]
  denom <- (2*exp((mod$estimates$log_omega[s]))^2)
  
  for (i in seq_along(x)) {
    for (j in seq_along(eldd)) {
      xx[i,j] <- wbar_full1(x[i], eldd[j], eta.part, mod$estimates$beta_clutchsize[s], num, denom, exp(mod$estimates$log_tau[s]), exp(mod$estimates$log_kappa[s]) , exp(data$offset[s]))
    }
  } 
  contour(x,eldd, xx, ylab=if (s==1) "Egg-laying date" else NULL,xlab="Clutch size", frame=F,axes=F,col=if (s==1) "red" else "green")
  axis(1)
  if (s==1) {
    axis.date();
  }  else {axis.date(labels=FALSE);
  }
}


# Function adding axis with March 1,April 1, etc as labels 
axis.date <- function(side=2,labels=TRUE) {
  at1 <- cumsum(c(0,31,30,31))
  if (labels) 
    labels <- c("March 1","April 1","May 1","June 1")
  axis(side, at=at1, labels=labels)
  at2 <- as.vector(outer(-1+at1,c(1,10,20,30),"+"))
  axis(side, at=at2, labels=FALSE, tcl=-.25)
}


rawdataplot <- function(mod,data,s) {
  dev.new(width=8,height=10.5)
  par(mfrow=c(7,5),mar=c(.3,.3,.1,.1))
  for (t in c(1979:1981,1983:2013)) {
    subset <- data$t==t
    z <- data$z[subset]
    y <- data$y[subset,]
    pobs <- y[,s+1]/y[,s]
    plot(y[,1], z,
         col=rgb(pobs,pobs,pobs),
         pch=16,cex=2,
         xlim=c(0,7), ylim=range(data$z))
    legend("topleft",legend=t,bty="n")
  }
}

# compute auto-covariance and autocorrelation matrix function of VAR(1) process given phi and rho
acmf <- function(phi,rho,lag.max=5) {
  lag0 <- lag.max + 1
  Phi <- matrix(phi,2,2,byrow=TRUE)
  Sigma <- toeplitz(c(1,rho))
  vecGamma0 <- solve(diag(4) - kronecker(Phi,Phi), as.vector(Sigma))
  Gamma <- array(dim=c(2, 2, 2*lag.max+1))
  Rho <- array(dim=c(2, 2, 2*lag.max+1))
  Gamma[,,lag0] <- matrix(vecGamma0, 2, 2)
  for (k in 1:lag.max) {
    Gamma[,,lag0 + k] <- Gamma[,,lag0 + k-1] %*% t(Phi)
    Gamma[,,lag0 - k] <- t(Gamma[,,lag0 + k])
  }
  isd <- 1/sqrt(diag(Gamma[,,lag0]))
  tmp <- list(Gamma=Gamma,Rho=isd * Gamma * rep(isd,each=2), lag.max=lag.max) 
  class(tmp) <- "acmf"
  tmp
}
plot.acmf <- function(x) {
  dim(x$Rho) <- c(4,2*x$lag.max + 1)
  matplot(t(x$Rho),lty=1:4,col="black",type="l")
}